@@ -11,7 +11,7 @@ function(global.model, cluster = NA, beta = FALSE, evaluate = TRUE,
 		.parallelPkgCheck() # XXX: workaround to avoid importing from 'parallel'
 		clusterCall <- get("clusterCall")
 		clusterApply <- get("clusterApply")
-		clusterCall(cluster, "require", "MuMIn", character.only = TRUE)
+		clusterCall(cluster, "require", .packageName, character.only = TRUE)
 		.getRow <- function(X) clusterApply(cluster, X, fun = ".pdredge_process_model")
 	} else {
 		.getRow <- function(X) lapply(X, pdredge_process_model, envir = props)
@@ -26,15 +26,16 @@ function(global.model, cluster = NA, beta = FALSE, evaluate = TRUE,
 	if (is.null(gmCall)) {
 		gmCall <- substitute(global.model)
 		if(!is.call(gmCall)) {
+			stop("need a 'global.model' with call component. Consider using ", 
 			if(inherits(global.model, c("gamm", "gamm4")))
-				message("for 'gamm' models use 'MuMIn::gamm' wrapper")
-			stop("could not retrieve the call to 'global.model'")
+					"'uGamm'" else "'updateable'")
 		}
 		#"For objects without a 'call' component the call to the fitting function \n",
 		#" must be used directly as an argument to 'dredge'.")
 		# NB: this is unlikely to happen:
-		if(!exists(as.character(gmCall[[1L]]), parent.frame(), mode = "function"))
-			 stop("could not find function '", gmCall[[1L]], "'")
+		if(!is.function(eval(gmCall[[1L]], parent.frame())))
+			gettext('could not find function "%s"', deparse(gmCall[[1L]], 
+				control = NULL), domain = "R")
 	} else {
 		# if 'update' method does not expand dots, we have a problem
 		# with expressions like ..1, ..2 in the call.
@@ -43,30 +44,39 @@ function(global.model, cluster = NA, beta = FALSE, evaluate = TRUE,
 		if(length(is.dotted) > 0L) {
 			substGmCall <- substitute(global.model)
 			if(is.name(substGmCall)) {
-				stop("call to 'global.model' contains '...' arguments and ",
-					"cannot be updated: ", deparse(gmCall, control = NULL))
+				.cry(NA, "call to 'global.model' contains '...' arguments and cannot be updated: %s", deparse(gmCall, control = NULL))
 			} else gmCall[is.dotted] <-
 				substitute(global.model)[names(gmCall[is.dotted])]
 		}
+		
 		## object from 'run.mark.model' has $call of 'make.mark.model' - fixing it here:
 		if(inherits(global.model, "mark") && gmCall[[1L]] == "make.mark.model") {
 			gmCall <- call("run.mark.model", model = gmCall, invisible = TRUE)
 		}
+		
 	}
 
     LL <- .getLik(global.model)
 	logLik <- LL$logLik
 	lLName <- LL$name
+	
 	# *** Rank ***
 	rank.custom <- !missing(rank)
+	
 	if(!rank.custom && lLName == "qLik") {
 		rank <- "QIC"
-		warning("using 'QIC' instead of 'AICc'")
+		.cry(NA, "using 'QIC' instead of 'AICc'", warn = TRUE)
 	}
+	
 	rankArgs <- list(...)
 	IC <- .getRank(rank, rankArgs)
 	ICName <- as.character(attr(IC, "call")[[1L]])
 
+	tryCatch(IC(global.model), error = function(e) {
+		e$call <- do.call(substitute, list(attr(IC, "call"), list(x = as.name("global.model"))))
+		stop(e)
+	})
+
 	allTerms <- allTerms0 <- getAllTerms(global.model, intercept = TRUE,
 		data = eval(gmCall$data, envir = gmEnv))
 
@@ -76,20 +86,18 @@ function(global.model, cluster = NA, beta = FALSE, evaluate = TRUE,
 	nInts <- sum(attr(allTerms, "intercept"))
 
 
-	
-
-
 	# parallel: check whether the models would be identical:
 	if(doParallel && check) testUpdatedObj(cluster, global.model, gmCall, level = check)
 
 	# Check for na.omit
-	if (!is.null(gmCall$na.action) &&
-		as.character(gmCall$na.action) %in% c("na.omit", "na.exclude")) {
-		stop("'global.model' should not use 'na.action' = ", gmCall$na.action)
-	}
+	if(!(gmNA.action <- .checkNaAction(cl = gmCall, what = "'global.model'")))
+		.cry(NA, attr(gmNA.action, "message"))
+	
 
 	if(names(gmCall)[2L] == "") gmCall <-
-		match.call(gmCall, definition = eval(gmCall[[1L]], envir = parent.frame()), expand.dots = TRUE)
+		match.call(gmCall, definition = eval(gmCall[[1L]], envir = parent.frame()),
+				   expand.dots = TRUE)
+
 
 
 	gmCoefNames <- fixCoefNames(names(coeffs(global.model)))
@@ -97,12 +105,12 @@ function(global.model, cluster = NA, beta = FALSE, evaluate = TRUE,
 	n.vars <- length(allTerms)
 
 	if(isTRUE(rankArgs$REML) || (isTRUE(.isREMLFit(global.model)) && is.null(rankArgs$REML)))
-		warning("comparing models fitted by REML")
+		.cry(NA, "comparing models fitted by REML", warn = TRUE)
 
 	if (beta && is.null(tryCatch(beta.weights(global.model), error = function(e) NULL,
 		warning = function(e) NULL))) {
-		warning("do not know how to calculate beta weights for ",
-				class(global.model)[1L], ", argument 'beta' ignored")
+		.cry(NA, "do not know how to calculate beta weights for '%s', argument 'beta' ignored",
+			 class(global.model)[1L], warn = TRUE)
 		beta <- FALSE
 	}
 
@@ -112,16 +120,16 @@ function(global.model, cluster = NA, beta = FALSE, evaluate = TRUE,
 	if (!is.null(fixed)) {
 		if (inherits(fixed, "formula")) {
 			if (fixed[[1L]] != "~" || length(fixed) != 2L)
-				warning("'fixed' should be a one-sided formula")
+				.cry(NA, "'fixed' should be a one-sided formula", warn = TRUE)
 			fixed <- as.vector(getAllTerms(fixed))
 		} else if (identical(fixed, TRUE)) {
 			fixed <- as.vector(allTerms[!(allTerms %in% interceptLabel)])
 		} else if (!is.character(fixed)) {
-			stop ("'fixed' should be either a character vector with"
-				  + " names of variables or a one-sided formula")
+			.cry(NA, paste("'fixed' should be either a character vector with",
+						   " names of variables or a one-sided formula"))
 		}
 		if (!all(fixed %in% allTerms)) {
-			warning("not all terms in 'fixed' exist in 'global.model'")
+			.cry(NA, "not all terms in 'fixed' exist in 'global.model'", warn = TRUE)
 			fixed <- fixed[fixed %in% allTerms]
 		}
 	}
@@ -174,7 +182,7 @@ function(global.model, cluster = NA, beta = FALSE, evaluate = TRUE,
 		applyExtras <- function(x) unlist(lapply(extra, function(f) f(x)))
 		extraResult <- applyExtras(global.model)
 		if(!is.numeric(extraResult))
-			stop("function in 'extra' returned non-numeric result")
+			.cry(NA, "function in 'extra' returned non-numeric result")
 
 		nextra <- length(extraResult)
 		extraNames <- names(extraResult)
@@ -187,8 +195,7 @@ function(global.model, cluster = NA, beta = FALSE, evaluate = TRUE,
 	nov <- as.integer(n.vars - n.fixed)
 	ncomb <- (2L ^ nov) * nvariants
 
-	if(nov > 31L) stop(gettextf("number of predictors (%d) exceeds allowed maximum (31)",
-								nov, domain = "R-MuMIn"))
+	if(nov > 31L) .cry(NA, "number of predictors (%d) exceeds allowed maximum (31)", nov)
 	#if(nov > 10L) warning(gettextf("%d predictors will generate up to %.0f combinations", nov, ncomb))
 	nmax <- ncomb * nvariants
 	if(evaluate) {
@@ -217,8 +224,9 @@ function(global.model, cluster = NA, beta = FALSE, evaluate = TRUE,
 			n <- length(allTerms)
 			if(is.null(dn) || any(sapply(dn, is.null))) {
 				di <- dim(subset)
-				if(any(di != n)) stop("unnamed 'subset' matrix does not have ",
-					"both dimensions equal to number of terms in 'global.model': %d", n)
+				if(any(di != n)) stop("unnamed 'subset' matrix does not have both dimensions",
+					" equal to number of terms in 'global.model': %d", n)
+
 				dimnames(subset) <- list(allTerms, allTerms)
 			} else {
 				if(!all(unique(unlist(dn)) %in% allTerms))
@@ -243,19 +251,50 @@ function(global.model, cluster = NA, beta = FALSE, evaluate = TRUE,
 			}
 			subset <- as.expression(subset)
 			ssValidNames <- c("comb", "*nvar*")
-			subsetExpr <- .subst4Vec(subset[[1L]], allTerms, as.name("comb"))
+
+			subsetExpr <- subset[[1L]]
+
+			## subset X
+			#gloFactorTable <- t(attr(terms(global.model), "factors")[-1L, ] != 0)
+			gloFactorTable <- t(attr(terms(reformulate(allTerms0[!(allTerms0
+				%in% interceptLabel)])), "factors") != 0)
+			
+			rownames(gloFactorTable) <- allTerms0[!(allTerms0 %in% interceptLabel)]
+	
+			
+			subsetExpr <- .substFun4Fun(subsetExpr, ".", function(x, fac, at, vName) {
+				if(length(x) != 2L) .cry(x, "exactly one argument needed, %d given.", length(x) - 1L)
+				if(length(x[[2L]]) == 2L && x[[2L]][[1L]] == "+") {
+					fun <- "all"
+					sx <- as.character(x[[2L]][[2L]])
+				} else {
+					fun <- "any"
+					sx <- as.character(x[[2L]])
+				}
+				#print(sx)
+				dn <- dimnames(fac)
+				#print(dn)
+				#browser()
+				if(!(sx %in% dn[[2L]])) .cry(x, "unknown variable name '%s'", sx)
+				as.call(c(as.name(fun), call("[", vName, as.call(c(as.name("c"), 
+					match(dn[[1L]][fac[, sx]], at))))))
+			}, gloFactorTable, allTerms, as.name("comb"))
+			
+			subsetExpr <- .subst4Vec(subsetExpr, allTerms, as.name("comb"))
+			
 
 			if(nvarying) {
 			ssValidNames <- c("cVar", "comb", "*nvar*")
 			subsetExpr <- .substFun4Fun(subsetExpr, "V", function(x, cVar, fn) {
-				if(length(x) > 2L)
-					warning("discarding extra arguments for 'V' in 'subset' expression")
+					if(length(x) > 2L) .cry(x, "discarding extra arguments", warn = TRUE)
 				i <- which(fn == x[[2L]])[1L]
-					if(is.na(i)) stop(sQuote(x[[2L]]), " is not a valid name of 'varying' element")
+					if(is.na(i)) .cry(x, "'%s' is not a valid name of 'varying' element",
+									  as.character(x[[2L]]), warn = TRUE)
 				call("[[", cVar, i)
 			}, as.name("cVar"), varying.names)
 			if(!all(all.vars(subsetExpr) %in% ssValidNames))
-				subsetExpr <- .subst4Vec(subsetExpr, varying.names, as.name("cVar"), fun = "[[")
+					subsetExpr <- .subst4Vec(subsetExpr, varying.names,
+											 as.name("cVar"), fun = "[[")
 			}
 			ssVars <- all.vars(subsetExpr)
 			okVars <- ssVars %in% ssValidNames
@@ -331,10 +370,12 @@ function(global.model, cluster = NA, beta = FALSE, evaluate = TRUE,
 	}
 	props <- as.environment(props)
 	
-	if(doParallel) clusterVExport(cluster,
-								  pdredge_props = props,
+	if(doParallel) {
+		clusterVExport(cluster,   pdredge_props = props,
 								  .pdredge_process_model = pdredge_process_model
 								  )
+		clusterCall(cluster, eval, call("options", options("na.action")), env = 0L)
+	}
 	# END parallel
 
 	retColIdx <- if(nvarying) -n.vars - seq_len(nvarying) else TRUE
@@ -348,6 +389,7 @@ function(global.model, cluster = NA, beta = FALSE, evaluate = TRUE,
 		if(jComb != prevJComb) {
 			isok <- TRUE
 			prevJComb <- jComb
+			
 			comb <- c(as.logical(intToBits(jComb - 1L)[comb.seq]), comb.sfx)
 			nvar <- sum(comb) - nInts
 			
@@ -518,7 +560,7 @@ function(global.model, cluster = NA, beta = FALSE, evaluate = TRUE,
 
 	ret <- structure(ret,
 		class = c("model.selection", "data.frame"),
-		calls = calls[o],
+		model.calls = calls[o],
 		global = global.model,
 		global.call = gmCall,
 		terms = structure(allTerms, interceptLabel = interceptLabel),
